package de.janschejbal.p2pbluetoothtest;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.bluetooth.BluetoothAdapter;
import android.content.ComponentName;
import android.content.Intent;
import android.content.pm.ApplicationInfo;
import android.os.Binder;
import android.os.Build;
import android.os.IBinder;
import android.support.v4.app.NotificationCompat;
import android.support.v4.app.NotificationCompat.Builder;
import android.util.Log;
import de.janschejbal.p2pbluetoothtest.DataCollectionManager.Status;
import de.janschejbal.p2pbluetoothtest.util.CryptoHelper;
import de.janschejbal.p2pbluetoothtest.util.FileUtils;
import de.janschejbal.p2pbluetoothtest.util.PRNGFixes;
import de.janschejbal.p2pbluetoothtest.util.FileUtils.FlagFile;

/**
 * This class represents a service that is always active, even when data collection is disabled.
 * 
 * Parts of this file were taken from TrackerService.java in Vespucci, written by me. 
 * @author Jan
 *
 */
public class BackgroundService extends Service {

	private Binder binder = new BackgroundServiceBinder();
	private boolean started = false;
	private String nodeID;
	private Builder notificationBuilder;

	private MainActivity mainActivity;
	private DataCollectionManager collectionManager;
	private LoggingManager loggingManager;
	private int version = -1;
	private String buildDate = "";
	
	private static final String NODE_ID_FILENAME = "node_id";
	private static final String BT_DISABLE_PREFIX = "disabled-";
	private final FlagFile flagIsCollecting = new FlagFile(this, "is_collecting");
	private final FlagFile flagKillswitch =  new FlagFile(this, "killswitch_enabled");
	private boolean cachedKillswitch = false;
	/** set to true on incompatble Android versions */
	private boolean versionKillswitch;
	private boolean bluetoothIsBuggy = false;
	static final String BLUETOOTH_120_SECOND_BUG_FLAG = "bluetooth_120_second_bug";

	@Override
	public void onCreate() {
		PRNGFixes.apply();
		super.onCreate();
		cachedKillswitch = flagKillswitch.get();
		
		versionKillswitch = !checkAndroidVersion();
		
		bluetoothIsBuggy = Build.VERSION.RELEASE.equals(FileUtils.readString(this, BLUETOOTH_120_SECOND_BUG_FLAG));
		
		// Fetch version info before initializing logging
		fetchVersionInfo();
		
		
		boolean isDebuggable =  ( 0 != ( getApplicationInfo().flags &= ApplicationInfo.FLAG_DEBUGGABLE ) );
		if (isDebuggable != CONFIG.USE_TESTNET) {
			Log.e("TESTNET MISMATCH", "Testnet/production version mismatch, killing app");
			throw new RuntimeException("Testnet/production version mismatch");
		}
		
		loggingManager = new LoggingManager(this);
		collectionManager = new DataCollectionManager(this, loggingManager);
	}


	/**
	 * Checks if the Android version should be supported
	 * @return true if yes, false if not
	 */
	private boolean checkAndroidVersion() {
		if (Build.VERSION.SDK_INT < Build.VERSION_CODES.GINGERBREAD_MR1) return false;
		// if (Build.VERSION.RELEASE.matches("^2\\.3\\.[345].*")) return false; // TODO-production enable
		return true;
	}


	private void fetchVersionInfo() {
		try {
			version = getPackageManager().getPackageInfo(getPackageName(), 0).versionCode;
			ApplicationInfo appinfo = getApplicationInfo();
			ZipFile apk = new ZipFile(appinfo.sourceDir);
			ZipEntry dex = apk.getEntry("classes.dex");
			buildDate  = new SimpleDateFormat("yyyy-MM-dd'T'HH:mmZ", Locale.US).format(new Date(dex.getTime()));
		} catch (Exception e) {
			Log.e("WTF", "Failed to fetch version info", e);
		}
	}
	
	
	@Override
	public int onStartCommand(Intent intent, int flags, int startId) {
		if (started) return START_STICKY;
		
		started = true;
		startForeground(R.id.notification_collection_service, getNotification());
		if (flagIsCollecting.get()) collectionManager.start();
		return START_STICKY;
	}
	
	
	@Override
	public void onDestroy() {
		collectionManager.stop();
		loggingManager.close();
		started = false;
		super.onDestroy();
	}
	
	public synchronized void startCollecting() {
		flagIsCollecting.set(true);
		collectionManager.start();
	}
	
	public synchronized void stopCollecting() {
		flagIsCollecting.set(false);
		collectionManager.stop();
		loggingManager.closeLog(); // just close (and possibly upload) the current log
	}
	
	
	
	private Notification getNotification() {
		if (notificationBuilder == null) {
			// initialize notification
			notificationBuilder = new NotificationCompat.Builder(this);
			Intent appStartIntent = new Intent();
			appStartIntent
				.setAction(Intent.ACTION_MAIN)
				.addCategory(Intent.CATEGORY_LAUNCHER)
				.setComponent(new ComponentName(MainActivity.class.getPackage().getName(), MainActivity.class.getName()))
				.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
			PendingIntent pendingAppIntent = PendingIntent.getActivity(this, 0, appStartIntent, 0);
			notificationBuilder
				.setContentIntent(pendingAppIntent)
				.setContentTitle("P2P-Datenerfassung")
//				.setShowWhen(false) // TODO enable when implemented in library
				.setOngoing(true);
		}
		
		if (getVersionKillswitch()) {
			return notificationBuilder
					.setSmallIcon(R.drawable.statusicon_red)
					.setContentText("Inkompatible Android-Version")
					.build();			
		}
		if (getKillswitch()) {
			return notificationBuilder
					.setSmallIcon(R.drawable.statusicon_red)
					.setContentText("App vom Server deaktiviert")
					.build();			
		}
		
		if (getServerMessageText() != null) {
			return notificationBuilder
					.setSmallIcon(R.drawable.statusicon_yellow)
					.setContentText("Systemnachricht vorhanden")
					.build();
		}
		
		if (loggingManager.getUploadOldest() < (System.currentTimeMillis() - CONFIG.UPLOAD_WARN_AGE)) {
			return notificationBuilder
					.setSmallIcon(R.drawable.statusicon_yellow)
					.setContentText("Upload notwendig")
					.build();
		}
		
		
		switch (collectionManager.getStatus()) {
		case STOPPED: return notificationBuilder
				.setSmallIcon(R.drawable.statusicon_disabled)
				.setContentText("Datenerfassung manuell deaktiviert")
				.build();
	
		case NO_BLUETOOTH: return notificationBuilder
				.setSmallIcon(R.drawable.statusicon_red)
				.setContentText("Bluetooth ist nicht aktiv")
				.build();
	
		case NOT_DISCOVERABLE: return notificationBuilder
				.setSmallIcon(bluetoothIsBuggy ? R.drawable.statusicon : R.drawable.statusicon_yellow)
				.setContentText("Bluetooth-Sichtbarkeit ist ausgeschaltet")
				.build();
	
		case NOT_NAMED_CORRECTLY: return notificationBuilder
				.setSmallIcon(R.drawable.statusicon_yellow)
				.setContentText("Bluetooth-Name falsch eingestellt")
				.build();
	
		case PERFECT: return notificationBuilder
				.setSmallIcon(R.drawable.statusicon)
				.setContentText("Alles in Ordnung")
				.build(); 
		}
		
		// All cases covered, but Java/Eclipse does not understand this, so:
		throw new RuntimeException("Unhandled status");
	}
	
	// TODO check for memleaks, or implement restart every 24 h
	
	@Override
	public IBinder onBind(Intent arg0) {
		startService(new Intent(this, BackgroundService.class));
		return binder;
	}
		
    public class BackgroundServiceBinder extends Binder {
        public BackgroundService getService() {
        	return BackgroundService.this;
        }
    }

	public void updateStatus() {
		NotificationManager notifier = (NotificationManager)getSystemService(NOTIFICATION_SERVICE);
		notifier.notify(R.id.notification_collection_service, getNotification());
		if (mainActivity != null) mainActivity.updateStatus();
	}


	public void setActivity(MainActivity mainActivity) {
		this.mainActivity = mainActivity;
	}

	public String getPublicNodeID() {
		return getNodeID().substring(0, 16);
	}

	public synchronized String getNodeID() {
		if (nodeID != null) return nodeID;
		
		// Try to read node ID
		nodeID = FileUtils.readString(this, NODE_ID_FILENAME);
		if (nodeID != null && nodeID.length() == 32) return nodeID;
		
		// No valid node ID on disk, we need to make a new one (16 byte = 32 hex characters)
		nodeID = CryptoHelper.randomHexBytes(16);
		FileUtils.writeString(this, NODE_ID_FILENAME, nodeID);
		return nodeID;
	}


	public Status getStatus() {
		return collectionManager.getStatus();
	}
	
	public LoggingManager getLoggingManager() {
		return loggingManager;
	}


	public int getVersion() {
		return version;
	}
	
	public String getBuildDate() {
		return buildDate;
	}

	public String getServerMessageUrl() {
		if (loggingManager == null) return null;
		return loggingManager.getServerMessageUrl();
	}
	
	public String getServerMessageText() {
		if (loggingManager == null) return null;
		return loggingManager.getServerMessageText();
	}


	public void setKillswitch(boolean state) {
		flagKillswitch.set(state);
		cachedKillswitch = state;
		if (state && collectionManager.isCollecting()) {
			Log.e("KILLSWITCH", "enabled");
			loggingManager.log("KILLSWITCH", "enabled");
			stopCollecting();
		}
		applyKillswitchToBluetooth();
		updateStatus();
	}
	
	public void applyKillswitchToBluetooth() {
		String name = BluetoothAdapter.getDefaultAdapter().getName();
		if (name == null) return;
		if (getKillswitch()) {
			if (name.startsWith(CONFIG.BT_NAME_PREFIX)) {
				BluetoothAdapter.getDefaultAdapter().setName(BT_DISABLE_PREFIX + getPublicNodeID());
			}
		} else {
			if (name.equals(BT_DISABLE_PREFIX + getPublicNodeID())) {
				BluetoothAdapter.getDefaultAdapter().setName(CONFIG.BT_NAME_PREFIX + getPublicNodeID());
			}
			
		}
	}


	public boolean getKillswitch() {
		return cachedKillswitch || versionKillswitch;
	}
	
	public boolean getVersionKillswitch() {
		return versionKillswitch;
	}

	public boolean isBluetoothBuggy() {
		return bluetoothIsBuggy;
	}
	
	public void setBluetoothBuggy() {
		bluetoothIsBuggy  = true;
		updateStatus();
	}

}
