package de.janschejbal.p2pbluetoothtest;

import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.BatteryManager;
import android.util.Log;
import de.janschejbal.p2pbluetoothtest.bluetooth.BluetoothManager;
import de.janschejbal.p2pbluetoothtest.broadcast.BroadcastManager;
import de.janschejbal.p2pbluetoothtest.util.AlarmHelper;
import de.janschejbal.p2pbluetoothtest.util.AlarmHelper.AlarmRunnable;

/**
 * A single instance of this class is kept by the {@link BackgroundService} for its entire lifetime.
 * This class can be started and stopped to start and stop data collection.
 * An instance of BluetoothCore will be started for each data collection run and persist for its duration.
 * 
 * @author Jan
 *
 */
public class DataCollectionManager {

	private static final String TAG = "DataCollectionManager";
	private boolean isCollecting;
	private BroadcastManager broadcastManager;
	private BluetoothManager bluetoothManager;
		
	private BackgroundService service;
	private Logger logger;
	private AlarmHelper batteryLoggingAlarm;
	
	
	public DataCollectionManager(BackgroundService owner, Logger logger) {
		this.service = owner;
		this.logger = logger;
	}
	
	public synchronized void start() {
		if (isCollecting) return;
		
		if (!ConsentActivity.hasConsent(service)) {
			Log.e(TAG, "Cannot start, no consent!");
			return;
		}
		
		if (service.getKillswitch()) {
			Log.e(TAG, "Cannot start, killswitch enabled!");
			return;
		}
		
		isCollecting = true;
		
		logger.log(TAG, "Starting collection");
		logger.log("CONSENT", ConsentActivity.getConsentInfo(service));
		
		broadcastManager = new BroadcastManager(service, logger);
		bluetoothManager = new BluetoothManager(service, broadcastManager, logger);
		
		batteryLoggingAlarm = new AlarmHelper(service);
		batteryLoggingAlarm.setRepeatingAlarm(false, 0L, CONFIG.BATTERY_LOG_INTERVAL, new AlarmRunnable() {
			@Override
			public void handleAlarm(int count) {
				IntentFilter ifilter = new IntentFilter(Intent.ACTION_BATTERY_CHANGED);
				Intent batteryStatus = service.registerReceiver(null, ifilter); // fetch sticky intent
				int status = batteryStatus.getIntExtra(BatteryManager.EXTRA_STATUS, -1);
				int level = batteryStatus.getIntExtra(BatteryManager.EXTRA_LEVEL, -1);
				int scale = batteryStatus.getIntExtra(BatteryManager.EXTRA_SCALE, -1);
				logger.log("BATT", status, level, scale);
			}
		});
		
		service.updateStatus();
	}
	
	public synchronized void stop() {
		if (!isCollecting) return;
		isCollecting = false;
		
		batteryLoggingAlarm.close();
		
		logger.log(TAG, "Stopping collection");
		
		bluetoothManager.close();
		bluetoothManager = null;

		broadcastManager.close();
		broadcastManager = null;
		
		service.updateStatus();
	}

	public boolean isCollecting() {
		return isCollecting;
	}


	public String getNodeID() {
		return service.getNodeID();
	}

	public String getPublicNodeID() {
		return service.getPublicNodeID();
	}

	public Context getContext() {
		return service;
	}

	public Logger getLogger() {
		return logger;
	}
	
	public enum Status {
		STOPPED,
		NO_BLUETOOTH,
		NOT_DISCOVERABLE,
		NOT_NAMED_CORRECTLY,
		PERFECT
	}

	public Status getStatus() {
		if (!isCollecting || bluetoothManager == null) return Status.STOPPED;
		return bluetoothManager.getStatus();
	}

}
