package de.janschejbal.p2pbluetoothtest.bluetooth;

import android.bluetooth.BluetoothAdapter;
import android.content.Context;
import android.content.Intent;
import de.janschejbal.p2pbluetoothtest.BackgroundService;
import de.janschejbal.p2pbluetoothtest.CONFIG;
import de.janschejbal.p2pbluetoothtest.DataCollectionManager.Status;
import de.janschejbal.p2pbluetoothtest.Logger;
import de.janschejbal.p2pbluetoothtest.protocol.base.ConnectionFactory;
import de.janschejbal.p2pbluetoothtest.util.SimpleReceiver;

/**
 * This class persists for the duration of one data collection session.
 * It checks if Bluetooth is enabled/disabled, reports the status,
 * and creates/destroys an instance of the BluetoothCore when Bluetooth is enabled/disabled.
 * 
 * @author Jan
 *
 */
public class BluetoothManager extends SimpleReceiver {
	
	private final static String TAG = BluetoothManager.class.getSimpleName();
	private final BackgroundService service;
	private final ConnectionFactory connectionFactory;
	private final Logger logger;

	private BluetoothAdapter bluetooth;
	private BluetoothCore bluetoothCore = null;
	private Status status = Status.NO_BLUETOOTH;
	

	public BluetoothManager(final BackgroundService service, ConnectionFactory connectionFactory, Logger logger) {
		super(service,
				BluetoothAdapter.ACTION_STATE_CHANGED,
				BluetoothAdapter.ACTION_SCAN_MODE_CHANGED,
				BluetoothAdapter.ACTION_LOCAL_NAME_CHANGED
			);
		
		this.service = service;
		this.connectionFactory = connectionFactory;
		this.logger = logger;
				
		this.bluetooth = BluetoothAdapter.getDefaultAdapter();

		logger.log(TAG, "starting");
		updateState();
	}
	
	private void updateState() {
		if (bluetooth.getState() != BluetoothAdapter.STATE_ON) {
			setStatus(Status.NO_BLUETOOTH);
			return;
		}

		if (bluetooth.getScanMode() != BluetoothAdapter.SCAN_MODE_CONNECTABLE_DISCOVERABLE) {
			setStatus(Status.NOT_DISCOVERABLE);
			return;
		}
		
		if (!bluetooth.getName().equals(CONFIG.BT_NAME_PREFIX+service.getPublicNodeID())) {
			setStatus(Status.NOT_NAMED_CORRECTLY);
			return;
		}
		
		setStatus(Status.PERFECT);		
	}

	
	private void setStatus(Status status) {
		logger.log(TAG, "setStatus", status.toString(), this.status.toString());
		service.applyKillswitchToBluetooth();
		if (this.status == status) return;
		
		boolean startBluetooth = this.status == Status.NO_BLUETOOTH && status != Status.NO_BLUETOOTH;
		boolean stopBluetooth  = this.status != Status.NO_BLUETOOTH && status == Status.NO_BLUETOOTH;
		
		this.status = status;
		service.updateStatus();
		
		if (startBluetooth) startBluetooth();
		if (stopBluetooth)  stopBluetooth();
	}
	
	private void startBluetooth() {
		logger.log(TAG, "startBluetooth");
		if (bluetoothCore != null) throw new RuntimeException("Tried to start bluetooth core twice");
		bluetoothCore = new BluetoothCore(service, connectionFactory, logger);
	}
	
	private void stopBluetooth() {
		logger.log(TAG, "stopBluetooth");
		if (bluetoothCore != null) bluetoothCore.close();
		bluetoothCore = null;
	}

	@Override
	public void onReceive(Context context, Intent intent) {
		
		if (intent.getAction().equals(BluetoothAdapter.ACTION_STATE_CHANGED)) {
			logger.log(TAG, "Bluetooth action", "STATE_CHANGED");	
		} else if (intent.getAction().equals(BluetoothAdapter.ACTION_SCAN_MODE_CHANGED)) {
			logger.log(TAG, "Bluetooth action", "SCAN_MODE_CHANGED");						
		} else if (intent.getAction().equals(BluetoothAdapter.ACTION_LOCAL_NAME_CHANGED)) {
			logger.log(TAG, "Bluetooth action", "LOCAL_NAME_CHANGED");									
		} else {
			return;
		}
		
		// Something about the adapter changed. Re-check if we can work.
		updateState();
	}
	
	
	
	public Status getStatus() {
		return status;
	}

	public void close() {
		super.close();
		logger.log(TAG, "closing");
		stopBluetooth();
	}

}
