package de.janschejbal.p2pbluetoothtest.broadcast;

import java.util.HashMap;

import android.bluetooth.BluetoothSocket;
import de.janschejbal.p2pbluetoothtest.BackgroundService;
import de.janschejbal.p2pbluetoothtest.CONFIG;
import de.janschejbal.p2pbluetoothtest.Logger;
import de.janschejbal.p2pbluetoothtest.protocol.base.ConnectionFactory;
import de.janschejbal.p2pbluetoothtest.protocol.base.ProtocolConnection;
import de.janschejbal.p2pbluetoothtest.protocol.base.ProtocolConnection.CloseObserver;
import de.janschejbal.p2pbluetoothtest.protocol.broadcast.BroadcastProtocolConnection;
import de.janschejbal.p2pbluetoothtest.util.AlarmHelper;
import de.janschejbal.p2pbluetoothtest.util.AlarmHelper.AlarmRunnable;
import de.janschejbal.p2pbluetoothtest.util.CryptoHelper;

public class BroadcastManager implements ConnectionFactory, AlarmRunnable {
	private final static String TAG = BroadcastManager.class.getSimpleName();
	private Logger logger;
	
	private final BroadcastState state;
	private final long myId;
	private int version;
	private int counter = 0;
	private final AlarmHelper alarm;
	
	/** Stores information about last seen peers */
	private HashMap<Long, PeerInfo> peerMap = new HashMap<Long, BroadcastManager.PeerInfo>();
	
	public BroadcastManager(BackgroundService service, Logger logger) {
		this.logger = logger;
		
		version = service.getVersion();
		
		state = new BroadcastState(service, logger);
		myId = CryptoHelper.hexToLong(service.getPublicNodeID());
		
		alarm = new AlarmHelper(service);
		alarm.setRepeatingAlarm(false, 0, CONFIG.BROADCAST_MESSAGE_CREATION_INTERVAL, this);
	}

	@Override
	public void handleAlarm(int repeats) {
		if (repeats > 100) {
			logger.log(TAG, "NOT creating messages - alarm count too high", repeats);
			return;
		}
		
		for (int i = 0; i < repeats; i++) {
			ChannelState myChannelState = state.getChannel(myId);
			int nextMessagId;
			if (myChannelState == null) {
				nextMessagId = 0;
			} else {
				int[] currentMessages = myChannelState.getMessageIDs();
				nextMessagId = currentMessages[currentMessages.length-1] + 1;
			}
			
			logger.log(TAG, "creating message", nextMessagId);
			state.addMessage(myId, nextMessagId);
			
			++counter;
		}
		
		if (counter >= CONFIG.BROADCAST_SAVE_EVERY) {
			counter = 0;
			state.save();
		}
	}
	
	public void close() {
		alarm.close();
		state.save();
	}

	public BroadcastState getState() {
		return state;
	}

	public long getMyId() {
		return myId;
	}

	@Override
	public ProtocolConnection getConnection(boolean isInitiator, BluetoothSocket socket, Logger logger, CloseObserver closeObserver) {
		return new BroadcastProtocolConnection(isInitiator, socket, logger, closeObserver, this);
	}

	public synchronized void completedTransaction(long remoteNodeId, int instanceId, long timestamp) {
		peerMap.put(remoteNodeId, new PeerInfo(instanceId, timestamp));
	}
	
	public synchronized PeerInfo getPeerInfo(long remoteNodeId) {
		PeerInfo p = peerMap.get(remoteNodeId);
		return p != null ? p : new PeerInfo(0,0);
	}
	
	public int getVersion() {
		return version;
	}

	public class PeerInfo {
		public final int instanceId;
		public final long timestamp;
		public PeerInfo(int instanceId, long timestamp) {
			this.instanceId = instanceId;
			this.timestamp  = timestamp;
		}
	}
}
