package de.janschejbal.p2pbluetoothtest.util;

import java.io.Closeable;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.Charset;

import android.content.Context;

public abstract class FileUtils {
	
	private static final Charset UTF8 = Charset.forName("UTF-8");

	/**
	 * Closes the given Closeable, ignoring all exceptions.
	 * @param stream Closeable to close, can safely be null
	 */
	public static void closeSilent(Closeable stream) {
    	try {
    		stream.close();
    	} catch (Exception e) {
    		// ignore both null pointer and IO exceptions
    	}
    }

	/**
	 * Write a String to a file as UTF-8
	 * @param ctx a Context
	 * @param filename the file name (relative to the context's file directory)
	 * @param stringToWrite the String to be written
	 * @return true on success, false on failure
	 */
	public static boolean writeString(Context ctx, String filename, String stringToWrite) {
		try {
			FileOutputStream out = ctx.openFileOutput(filename, Context.MODE_PRIVATE);
			out.write(stringToWrite.getBytes(UTF8));
			out.close();
			return true;
		} catch (Exception e) {
			return false;				
		}
	}

	/**
	 * Reads the entire input stream as an UTF-8 string, closing it afterwards
	 * @param in stream to read
	 * @return String representation of stream contents, or null on error
	 */
	public static String readString(InputStream in) {
		char[] buf = new char[1*1024];
		int charsRead = 0;

		StringBuilder str = new StringBuilder();
		InputStreamReader reader = new InputStreamReader(in, Charset.defaultCharset()); // Default charset is always utf8 on android
		
		try {
			while ((charsRead = reader.read(buf)) != -1) {
				str.append(new String(buf, 0, charsRead));
			}
		} catch (IOException e) {
			return null;
		} finally {
			FileUtils.closeSilent(reader);
		}
		
		return str.toString();    	
	}

	/**
	 * Reads the entire file as an UTF-8 string, closing it afterwards.
	 * File path is relative context's files directory.
	 * @param ctx a Context
	 * @param filename the relative file name
	 * @return the file contents, or null on error
	 */
	public static String readString(Context ctx, String filename) {
		try {
			return readString(new FileInputStream(new File(ctx.getFilesDir(), filename)));
		} catch (IOException e) {
			return null;
		}
	}
	
	public static class FlagFile {
		
		private Context context;
		private String filename;

		public FlagFile(Context context, String filename) {
			this.context = context;
			this.filename = filename;
		}
		
		public synchronized void set(boolean state) {
			if (state) {
				// Set the flag
				try { context.openFileOutput(filename, Context.MODE_PRIVATE).close(); } catch (Exception e) {}
			} else {
				// Unset the flag
				new File(context.getFilesDir(), filename).delete();
			}
		}
		
		public synchronized boolean get() {
			return new File(context.getFilesDir(), filename).isFile();
		}
	}
}
