package de.janschejbal.p2pbluetoothtest.bluetooth;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.LinkedBlockingQueue;

import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.os.Handler;
import android.os.Handler.Callback;
import android.os.Message;
import de.janschejbal.p2pbluetoothtest.CONFIG;
import de.janschejbal.p2pbluetoothtest.Logger;
import de.janschejbal.p2pbluetoothtest.protocol.base.ConnectionFactory;
import de.janschejbal.p2pbluetoothtest.protocol.base.ProtocolConnection;
import de.janschejbal.p2pbluetoothtest.util.FileUtils;

/**
 * This thread handles establishing outgoing connections.
 */
public class BluetoothClientThread extends Thread implements Callback {
	public final static String TAG = BluetoothClientThread.class.getSimpleName();
	
	private boolean run = true;
	private boolean killConnections = false;

	private LinkedBlockingQueue<BluetoothDevice> connectionList = new LinkedBlockingQueue<BluetoothDevice>();
	private BluetoothCore core;
	private Logger logger;

	private BluetoothDevice currentDevice = null;
	private BluetoothSocket currentSocket = null;
	private ProtocolConnection currentConnection = null;
	
	private Handler timeoutHandler = new Handler(this);

	private final ConnectionFactory connectionFactory;

	
	public BluetoothClientThread(ConnectionFactory connectionFactory, BluetoothCore core, Logger logger) {
		this.connectionFactory = connectionFactory;
		this.core = core;
		this.logger = logger;
		
		start();
	}
	
	@Override
	public void run() {
		logger.log(TAG, "starting");
		while (run) {
			interrupted(); // clear possible interrupt flags
			try {
				if (killConnections || currentConnection == null || currentConnection.isClosed()) {
					closeConnections();
					killConnections = false;
				}
				
				if (currentConnection != null) {
					// waiting for current connection to finish
					synchronized (currentConnection) {
						currentConnection.wait(100);
					}
				} else {
					// No (active) current connection, wait until there is something to connect to, and connect to it.
					if (connectionList.peek() == null) core.clientConnectionsDone();
					currentDevice = connectionList.take();
					
					if (core.serverConnectedTo(currentDevice.getAddress())) {
						logger.log(TAG, "skipping device (already connected to server)", currentDevice.getName());
						continue;
					}
					
					logger.log(TAG, "creating socket", currentDevice.getName());
					
					// Set timeout
					timeoutHandler.sendEmptyMessageDelayed(0, CONFIG.CLIENT_CONNECTION_TIMEOUT);
					
					// This happens near-instantly and reliably; the real action happens in connect()
					currentSocket = currentDevice.createInsecureRfcommSocketToServiceRecord(BluetoothCore.BT_SERVICE_UUID);
					
					
					logger.log(TAG, "connecting", currentDevice.getName());
					
					// If you forget this, you are going to have a lot of fun with the debugger.
					// The resulting exceptions are utterly useless.
					currentSocket.connect();
					
					// Clear timeout
					timeoutHandler.removeCallbacksAndMessages(null);
					
					currentConnection = connectionFactory.getConnection(true, currentSocket, logger, null);
				}
			} catch (InterruptedException e) {
				// We were interrupted. That is OK, just re-run the loop iteration to react to new state.
			} catch (Exception e) {
				logger.log(TAG, "exception while connecting", e.toString());
			}
			
			// Synchronized to make sure that if a timeout handler is executed,
			// both the closing of the socket and interrupt() have run before the next loop iteration
			synchronized (timeoutHandler) {
				timeoutHandler.removeCallbacksAndMessages(null); // Stop timeout after interrupt/exception
			}
		}
		logger.log(TAG, "shutting down");
		closeConnections();
	}
	
	/**
	 * Closes and nulls any sockets/connections that may be open
	 */
	private void closeConnections() {
		logger.log(TAG, "closeConnections");
		FileUtils.closeSilent(currentSocket);
		FileUtils.closeSilent(currentConnection);
		currentSocket = null;
		currentConnection = null;
	}
	
	/**
	 * Starts connecting to the devices provided, in an order chosen by this thread.
	 * The thread may choose to skip some devices.
	 * @param targets a list of available devices
	 */
	public void startConnecting(List<BluetoothDevice> targets) {
		logger.log(TAG, "startConnecting", targets.size());		
		targets = new ArrayList<BluetoothDevice>(targets);
		Collections.shuffle(targets); // TONOTDO more intelligent selection?
		connectionList.addAll(targets);
	}
	
	/**
	 * Stops creating new connections, but keeps existing ones.
	 */
	public void stopConnecting() {
		logger.log(TAG, "stopConnecting");
		connectionList.clear();
	}
	
	/**
	 * Immediately closes any active connections, but keeps the thread running for later use.
	 */
	public void stopCommunication() {
		logger.log(TAG, "stopCommuication");
		stopConnecting();
		killConnections = true;
		interrupt();
	}
	
	/**
	 * Shuts down the thread, closing all connections
	 */
	public void close() {
		logger.log(TAG, "close");
		run = false;
		interrupt();
	}

	/**
	 * Handle connection timeouts
	 */
	@Override
	public boolean handleMessage(Message msg) {
		logger.log(TAG, "timeout");

		synchronized (timeoutHandler) { // Ensure this gets executed fully before the next run loop is executed
			FileUtils.closeSilent(currentSocket);
			
			// Does not interrupt connection attempt reliably on all devices.
			// (NOTE: This was before the closing of currentSocket was implemented,
			// which is how connection attempts should be interrupted)
			interrupt();
		}
		
		return true;
	}



}
