package de.janschejbal.p2pbluetoothtest;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.bluetooth.BluetoothAdapter;
import android.content.ComponentName;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.ServiceConnection;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.IBinder;
import android.util.Log;
import android.view.Menu;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;
import de.janschejbal.p2pbluetoothtest.DataCollectionManager.Status;
import de.janschejbal.p2pbluetoothtest.util.FileUtils;
import de.janschejbal.p2pbluetoothtest.util.PRNGFixes;

public class MainActivity extends Activity implements ServiceConnection {

	private static final int REQUESTCODE_BT = 10;
	private BackgroundService backgroundService;
	private View mainView;
   

	@Override
    public void onCreate(Bundle savedInstanceState) {
		PRNGFixes.apply();
        super.onCreate(savedInstanceState);
        mainView = View.inflate(this, R.layout.activity_main, null);
        setUpButtons();
        setContentView(mainView);
    }
    
    private void setUpButtons() {
		mainView.findViewById(R.id.button_stop).setOnClickListener(buttonActionStop);
		mainView.findViewById(R.id.button_msg).setOnClickListener(buttonActionMessage);		
		mainView.findViewById(R.id.button_weblink).setOnClickListener(buttonActionWeblink);		
	}
    
	@Override
    protected void onResume() {
    	
    	bindService(new Intent(this, BackgroundService.class), this, BIND_AUTO_CREATE);
    	
    	super.onResume();
    	
    	if (!ConsentActivity.hasConsent(this)) {
    		finish();
    		startActivity(new Intent(this, ConsentActivity.class));
    	}
    	
    	updateStatus();
    }
	
	@Override
	protected void onPause() {
		super.onPause();
	}
    
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.activity_main, menu);
        return true;
    }

	public void onServiceConnected(ComponentName name, IBinder service) {
		BackgroundService.BackgroundServiceBinder binder = (BackgroundService.BackgroundServiceBinder) service;
		Log.i("MainActivity", "Service connected");
		backgroundService = binder.getService();
		backgroundService.setActivity(this);
		updateStatus();
		
		if (getIntent().getBooleanExtra(ConsentActivity.EXTRA_CONSENT_GIVEN, false)) {
			getIntent().removeExtra(ConsentActivity.EXTRA_CONSENT_GIVEN);
			buttonActionStart.onClick(null);
		}
		((TextView)findViewById(R.id.textInfo)).setText("Version: " + backgroundService.getVersion() + ", " +
				"Node " + CONFIG.BT_NAME_PREFIX + backgroundService.getPublicNodeID());
	}

	public void onServiceDisconnected(ComponentName name) {}

	@Override
	protected void onDestroy() {
		try {
			unbindService(this);
		} catch (Exception e) {} // ignore errors, this is just cleanup
		super.onDestroy();
	}

	public void updateStatus() {
		Button buttonStart = (Button)mainView.findViewById(R.id.button_start);
		Button buttonStop = (Button)mainView.findViewById(R.id.button_stop);
		Button buttonUpload = (Button)mainView.findViewById(R.id.button_upload);
		Button buttonMessage = (Button)mainView.findViewById(R.id.button_msg);
		
		if (backgroundService == null) {
			buttonStart.setEnabled(false);
			buttonStart.setText("Initialisiere...");
			buttonStart.setVisibility(View.VISIBLE);
			buttonStop.setVisibility(View.GONE);
			buttonUpload.setVisibility(View.GONE);
			return;
		}
		
		buttonStart.setVisibility(backgroundService.getStatus() == Status.PERFECT ? View.GONE : View.VISIBLE);
		buttonStop.setVisibility(backgroundService.getStatus() == Status.STOPPED ? View.GONE : View.VISIBLE);
		buttonUpload.setVisibility(View.VISIBLE);

		if (backgroundService.getKillswitch()) {
			buttonStart.setEnabled(false);
			buttonStart.setText(backgroundService.getVersionKillswitch() ? buttonLabelStartWrongAndroid : buttonLabelStartKilled);
		} else {
			buttonStart.setEnabled(true);
		
			switch (backgroundService.getStatus()) {
			case STOPPED:
				buttonStart.setText(buttonLabelStart);
				buttonStart.setOnClickListener(buttonActionStart);
				break;
			case NO_BLUETOOTH:
			case NOT_DISCOVERABLE:
				buttonStart.setText(buttonLabelBluetooth);
				buttonStart.setOnClickListener(buttonActionBluetooth);
				break;
			case NOT_NAMED_CORRECTLY:
				buttonStart.setText(buttonLabelBluetoothName);
				buttonStart.setOnClickListener(buttonActionBluetoothName);
				break;
			default:
				break;		
			}
		}
			
		int uploadCount = backgroundService.getLoggingManager().getUploadCount();
		int uploadKB = (int)Math.ceil((double)backgroundService.getLoggingManager().getUploadSize()/1024.0);
		buttonUpload.setOnClickListener(buttonActionUpload);
		if (backgroundService.getLoggingManager().isUploading()) {
			buttonUpload.setText(String.format(buttonLabelUploadRunning, uploadCount, uploadKB));
			buttonUpload.setEnabled(false);
		} else {
			buttonUpload.setText(String.format(buttonLabelUpload, uploadCount, uploadKB));
			buttonUpload.setEnabled(true);
		}
		
		
		final String msgUrl = backgroundService.getServerMessageUrl();
		final String msgText = backgroundService.getServerMessageText();
		buttonMessage.setText(msgText != null ? msgText : buttonLabelMessage);
		buttonMessage.setEnabled(msgUrl != null);
	}

	private void requestBluetooth() {
		if (BluetoothAdapter.getDefaultAdapter().getScanMode() == BluetoothAdapter.SCAN_MODE_CONNECTABLE_DISCOVERABLE) {
			setBluetoothName();
			return;
		}
		Intent discoverableIntent = new	Intent(BluetoothAdapter.ACTION_REQUEST_DISCOVERABLE);
		discoverableIntent.putExtra(BluetoothAdapter.EXTRA_DISCOVERABLE_DURATION, 0);
		startActivityForResult(discoverableIntent, REQUESTCODE_BT);
	}
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		if (requestCode == REQUESTCODE_BT) {
			if (resultCode != RESULT_CANCELED) {
				setBluetoothName();
			}
			//  erkennen, ob android-version broken (120 sekunden timeout)
			//  wenn ja - flag setzen (wird beim nchsten log-start geloggt) und ohne sichtbarkeit arbeiten
			Log.d("BTRESULTCODE", ""+resultCode);
			if (resultCode == 120) {
				markBluetoothBroken();
			}
		}
	}
	
	private void setBluetoothName() {
		if (backgroundService == null || backgroundService.getKillswitch()) return;
		BluetoothAdapter.getDefaultAdapter().setName(CONFIG.BT_NAME_PREFIX + backgroundService.getPublicNodeID());
	}
	
	private void markBluetoothBroken() {
		FileUtils.writeString(this, BackgroundService.BLUETOOTH_120_SECOND_BUG_FLAG, Build.VERSION.RELEASE);
		if (backgroundService != null) backgroundService.setBluetoothBuggy();
	}
	
	
	//////////////////////// 
	/// Button stuff
	


	private String buttonLabelStart = "Datenerfassung starten";
	private String buttonLabelStartKilled = "DATENERFASSUNG VOM SERVER DEAKTIVIERT - BITTE VERSUCHSLEITER KONTAKTIEREN";
	private String buttonLabelStartWrongAndroid = "DATENERFASSUNG DEAKTIVIERT - inkompatible Android-Version. Bitte falls mglich das System aktualisieren.";
	private OnClickListener buttonActionStart = new OnClickListener() {
		public void onClick(View v) {
			if (backgroundService == null) return;
			if (backgroundService.getKillswitch()) return;
			backgroundService.startCollecting();
			requestBluetooth();
		}
	};
	
	private OnClickListener buttonActionStop = new OnClickListener() {
		public void onClick(View v) {
			if (backgroundService == null) return;
			backgroundService.stopCollecting();
			
			if (BluetoothAdapter.getDefaultAdapter().getState() != BluetoothAdapter.STATE_ON) return;
			
			boolean discoverable = (BluetoothAdapter.getDefaultAdapter().getScanMode() == BluetoothAdapter.SCAN_MODE_CONNECTABLE_DISCOVERABLE);
			
			// show dialog to ask if the user wants to change bluetooth settings 
			String text = discoverable ? "Bluetooth und Sichtbarkeit sind noch aktiv" :  "Bluetooth ist noch aktiv";
			new AlertDialog.Builder(MainActivity.this)
				.setTitle("Hinweis")
				.setMessage(text)
				.setPositiveButton("OK", null)
				.setNeutralButton("Bluetooth-Einstellungen", new Dialog.OnClickListener() {
						@Override
						public void onClick(DialogInterface dialog, int which) {
						    Intent intentBluetooth = new Intent();
						    intentBluetooth.setAction(android.provider.Settings.ACTION_BLUETOOTH_SETTINGS);
						    startActivity(intentBluetooth);
						}
					})
				.show();
		}
	};

	private String buttonLabelBluetooth = "Bluetooth konfigurieren";
	private OnClickListener buttonActionBluetooth = new OnClickListener() {
		public void onClick(View v) {
			requestBluetooth();
		}
	};
	
	private String buttonLabelBluetoothName = "Bluetooth-Namen setzen";
	private OnClickListener buttonActionBluetoothName = new OnClickListener() {
		public void onClick(View v) {
			setBluetoothName();
		}
	};
	
	// Upload button
	private String buttonLabelUpload = "Upload (%d Protokolle, %d KB)";
	private String buttonLabelUploadRunning = "Upload luft... (%d Protokolle, %d KB)";
	private OnClickListener buttonActionUpload = new OnClickListener() {
		public void onClick(View v) {
			if (backgroundService == null) return;
			if (backgroundService.getLoggingManager().hasWifi()) {
				backgroundService.getLoggingManager().triggerUpload(false);
			} else {
				AlertDialog.Builder dialog = new AlertDialog.Builder(MainActivity.this);
				dialog.setTitle("Upload erzwingen?");
				dialog.setMessage("Kein WLAN verfgbar. Soll trotzdem versucht werden, " + 
							(backgroundService.getLoggingManager().getUploadSize())/1024 + "KB hochzuladen? " +
							"Je nach Mobilfunkvertrag knnen dafr Kosten anfallen!"
						);
				dialog.setPositiveButton("Ja", new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int which) {
						if (backgroundService != null) {
							backgroundService.getLoggingManager().triggerUpload(true);
						}
					}
				});
				dialog.setNegativeButton("Nein", null);
				dialog.create().show();
			}
		}
	};
	
	private String buttonLabelMessage = "Keine Systemnachrichten";
	private OnClickListener buttonActionMessage  = new OnClickListener() {
		public void onClick(View v) {
			if (backgroundService == null) return;
			String msgUrl = backgroundService.getServerMessageUrl();
			if (msgUrl == null) return;
			try {
				startActivity(new Intent(Intent.ACTION_VIEW,Uri.parse(msgUrl)));
			} catch (Exception e) {
				Toast.makeText(MainActivity.this, "Fehler beim Aufrufen der URL", Toast.LENGTH_LONG).show();
			}
		}
	};
	
	private OnClickListener buttonActionWeblink  = new OnClickListener() {
		public void onClick(View v) {
			try {
				startActivity(new Intent(Intent.ACTION_VIEW,Uri.parse(CONFIG.ABOUT_URL)));
			} catch (Exception e) {
				Toast.makeText(MainActivity.this, "Fehler beim Aufrufen der URL", Toast.LENGTH_LONG).show();
			}
		}
	};
	

}
