package de.janschejbal.p2pbluetoothtest.broadcast;

import java.io.ByteArrayOutputStream;
import java.io.DataInput;
import java.io.DataOutput;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.Serializable;
import java.util.BitSet;

import de.janschejbal.p2pbluetoothtest.util.CryptoHelper;

/**
 * We are only interested in the most current n messages (n being defined by MAX_MESSAGES).
 * We store a bitmap showing which messages we have, and an offset defining the number of the leftmost message.
 * Thus, the range of interesting messages is oldestMessage..(oldestMessage+MAX_MESSAGES-1).
 * This is also the range represented by the bitmap.
 * 
 * Note that this class does not necessarily represent an actual state;
 * it may also be used to represent a list of messages.
 */
public class ChannelState implements Serializable {
	private static final long serialVersionUID = 1L;
	
	/** max. number of messages stored. must be divisible by 8. */
	public static final int MAX_MESSAGES = 512; // 64*8
	public static final int SERIALIZED_SIZE = 8 + 4 + MAX_MESSAGES/8;  // channelID, oldestMessage, messageBitmap

	private final long channelID;
	
	/** ID of the message represented by the leftmost bit in the messageBitmap */
	private int oldestMessage;
	
	/** Bitmap indicating which messages we have */
	private BitSet messageBitmap;
	

	public ChannelState(long channelID) {
		this.channelID = channelID;
		this.oldestMessage = 0;
		this.messageBitmap = new BitSet(MAX_MESSAGES);
	}
	
	/**
	 * Clone constructor - creates an independent instance with the same data as the original state.
	 * @param orig the ChannelState to clone
	 */
	public ChannelState(ChannelState orig) {
		this.channelID = orig.channelID;
		this.oldestMessage = orig.oldestMessage;
		this.messageBitmap = (BitSet)orig.messageBitmap.clone();
	}
	
	public ChannelState(DataInput reader) throws IOException {
		this.channelID = reader.readLong();
		this.oldestMessage = reader.readInt();
		this.messageBitmap = new BitSet(MAX_MESSAGES);
		byte[] bits = new byte[MAX_MESSAGES/8];
		reader.readFully(bits);
		for (int i = 0; i < MAX_MESSAGES; i++) {
			byte currbyte = (bits[i/8]); 
			int mask = (128 >> (i%8));
			if ((currbyte & mask) > 0) messageBitmap.set(i);
		}
	}
	
	/**
	 * Set the message with the given ID as available if possible.
	 * This will shift the bitmap if necessary (i.e. discard old messages).
	 * @param messageID the message to add
	 * @return true if the message was added, false if the message was not added (too old or already present)
	 */
	public synchronized boolean addMessage(int messageID) {
		int offset = shiftFor(messageID);
		if (offset < 0) return false;
		if (messageBitmap.get(offset)) return false;
		messageBitmap.set(offset);
		return true;
	}
	
	/**
	 * Shifts the messageBitmap to ensure messageID fits, if possible.
	 * 
	 * @param messageID
	 * @returnthe offset of messageID in the bitmap, or a negative value if the message is too old.
	 */
	private int shiftFor(int messageID) {
		int offset = messageID - oldestMessage;

		// Is the message older than the oldest message in our bitmap? If yes, it cannot be inserted.
		if (offset < 0) return -1;
		
		// Is the message in the current bitmap?
		if (offset < MAX_MESSAGES) return offset;
		
		// Message is newer than the area covered by our bitmap.
		// messageID will become the newest message, i.e. the offset will be MAX_MESSAGES-1
		// The oldest message will be messageID-(MAX_MESSAGES-1)
		shiftTo(messageID-(MAX_MESSAGES-1));
		return MAX_MESSAGES-1;
	}
	
	/**
	 * Shifts the bitmap left so oldestMessage matches the given new oldest message.
	 * If the given new oldest message is older than the current oldestMessage, this does nothing.
	 * @param newOldestMessage - the oldestMessage to match
	 * @return the object itself (for chaining)
	 */
	private ChannelState shiftTo(int newOldestMessage) {
		if (newOldestMessage <= oldestMessage) return this;
		
		int delta = newOldestMessage - oldestMessage;
		oldestMessage = newOldestMessage;		 
		messageBitmap = (delta > MAX_MESSAGES)
							? new BitSet(MAX_MESSAGES)
							: messageBitmap.get(delta, delta+MAX_MESSAGES);
		return this;
	}
	
	/**
	 * Creates a channelState representing the messages that are present in the left channel state,
	 * but not present in the right one.
	 * @param left
	 * @param right
	 * @return
	 */
	public static ChannelState getMissingRight(ChannelState left, ChannelState right) {
		// Normalize to the same oldestMessage. One of the shifts will be a no-op.
		ChannelState leftNormalized  = new ChannelState(left).shiftTo(right.oldestMessage);
		ChannelState rightNormalized = new ChannelState(right).shiftTo(left.oldestMessage);
		
		// Get messages we have and other does not have
		leftNormalized.messageBitmap.andNot(rightNormalized.messageBitmap);
		return leftNormalized;
	}
	
	/**
	 * @return An array of messageIDs of the messages that this ChannelState has
	 */
	public synchronized int[] getMessageIDs() {
		int[] result = new int[messageBitmap.cardinality()];
		int entry = 0;
		for (int i = 0; i < MAX_MESSAGES; i++) {
			if (messageBitmap.get(i)) {
				result[entry++] = oldestMessage + i;
			}
		}
		return result;
	}

	/**
	 * Writes a representation of the ChannelState object to the {@link DataOutput} 
	 * @param buf
	 */
	public synchronized void writeTo(DataOutput out) throws IOException {
		out.writeLong(channelID);
		out.writeInt(oldestMessage);
		out.write(getBitmapAsBytes());
	}
	
	public synchronized byte[] getBitmapAsBytes() {
		byte[] bytes = new byte[MAX_MESSAGES/8];
		for (int i = 0; i < MAX_MESSAGES; i++) {
			if (messageBitmap.get(i)) {
				bytes[i/8] |=  (128 >> (i%8));
			}
		}
		return bytes;
	}

	/**
	 * @return a compact string representation (hexdump of the serialized representation) 
	 */
	@Override
	public synchronized String toString() {
		byte[] buf;
		try {
			ByteArrayOutputStream bos = new ByteArrayOutputStream(SERIALIZED_SIZE);
			DataOutputStream dos = new DataOutputStream(bos);
			writeTo(dos);
			buf = bos.toByteArray();
			dos.close();
		} catch (Exception e) {
			return "ChannelState.toString() failed";
		}
		return CryptoHelper.bytesToHex(buf);
	}
	
	/**
	 * @return a verbose string representation (decimal padded oldestMessage, space, binary bitmap)
	 */
	public synchronized String toStringVerbose() {
		StringBuilder str = new StringBuilder(9+12+MAX_MESSAGES);
		str.append(CryptoHelper.longToHex(channelID));
		str.append(String.format(" %10d ", oldestMessage));
		for (int i = 0; i < MAX_MESSAGES; i++) {
			str.append(messageBitmap.get(i) ? '1' : '0');
		}
		return str.toString();
	}
	
	/**
	 * Compares this ChannelState with another.
	 * Returns true if both represent the same channelID, regardless of the inner state.
	 */
	@Override
	public boolean equals(Object o) {
		if (!(o instanceof ChannelState)) return false;
		return this.channelID == ((ChannelState)o).channelID;
	}

	public long getChannelId() {
		return channelID;
	}

	public synchronized boolean isEmpty() {
		return messageBitmap.cardinality() == 0;
	}

	public synchronized boolean hasMessage(int messageId) {
		int offset = messageId - oldestMessage;
		if (offset < 0 || offset >= MAX_MESSAGES) return false;
		return messageBitmap.get(offset);
	}
}
