package de.janschejbal.p2pbluetoothtest.protocol.broadcast;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;

import android.bluetooth.BluetoothSocket;
import android.os.SystemClock;
import de.janschejbal.p2pbluetoothtest.CONFIG;
import de.janschejbal.p2pbluetoothtest.Logger;
import de.janschejbal.p2pbluetoothtest.broadcast.BroadcastManager;
import de.janschejbal.p2pbluetoothtest.broadcast.BroadcastManager.PeerInfo;
import de.janschejbal.p2pbluetoothtest.broadcast.ChannelState;
import de.janschejbal.p2pbluetoothtest.protocol.base.ProtocolConnection;
import de.janschejbal.p2pbluetoothtest.protocol.base.ProtocolException;
import de.janschejbal.p2pbluetoothtest.protocol.base.SafeReader;
import de.janschejbal.p2pbluetoothtest.util.CryptoHelper;


/**
 * Protocol implementing a simple broadcast networks.
 * NOTE: This is a proof-of-concept implementation with limited error checking.
 * It assumes that the other node does not maliciously violate protocol,
 * and assumes there is only a reasonably limited number of nodes/channels (at most several thousand).
 * For example, no authentication of the node ID is make, allowing DoS using a false instanceId/timestamp.
 * 
 * Both nodes execute this protocol in parallel, i.e. there are two instances of this protocol in each connection.
 * Each node acts as Node A in one instance and Node B in the other.
 * (i.e. both nodes immediately send a HELLO message, respond with a STATE message to the HELLO they receive etc.)
 * 
 * 
 * <ol>
 * <li> {@link #MESSAGE_HELLO} - type {@link HelloMessage}
 *      Node A sends "Hello" message </li>
 *  
 * <li> {@link #MESSAGE_STATE} - type {@link ChannelMessage} 
 *      Node B sends either its full state list or the states that changed since the last completed exchange.
 *      (May be empty, but must be sent)</li>
 * 
 * <li> {@link #MESSAGE_REQUEST} - type {@link ChannelMessage}
 *      Node A requests the messages he is interested in. (May be empty, but must be sent)</li>
 *  
 * <li> {@link #MESSAGE_PAYLOAD}* - type {@link PayloadMessage}
 *      Node B fulfills the received request for messages by sending one payload messages per carried message</li>
 *    
 * <li> {@link #MESSAGE_END_OF_DATA} - type {@link EndOfDataMessage}
 *      Node B sends "End of data" message 
 *      This includes an instanceID and timestamp which may be used during the next connection to this node.</li>
 * <li> {@link #MESSAGE_FINISHED} - type {@link FinishedMessage}
 * 	    Node A confirms receipt of the "End of data" message.
 *      The connection can be closed as soon as one node has received both the end of data and finished messages,
 *      because that means that the node is done receiving data (end-of-data message)
 *      and the other node is also done receiving data (finished message)
 * </ol>
 */
public class BroadcastProtocolConnection extends ProtocolConnection {
	
	protected String getTag() { return "BroadcastProtocolConnection"; }

	private final BroadcastManager manager;
	/** used by {@link #oneSideFinished()} to remember that it has been called */ 
	private boolean oneSideFinished = false;
	private long remoteNodeId = 0;
	private int sentInstanceId = 0;
	private long sentTimestamp = 0;
	public BroadcastProtocolConnection(boolean isInitiator, BluetoothSocket socket,
			Logger logger, CloseObserver observer, BroadcastManager manager) {
		super(isInitiator, socket, logger, CONFIG.BROADCAST_TIMEOUT, observer);
		this.manager = manager;
		
		try {
			remoteNodeId = CryptoHelper.hexToLong(socket.getRemoteDevice().getName().substring(4));
		} catch (Exception e) {
			log("Failed to get remote node id");
			remoteNodeId = 0;
		}
		
		startProcessingThreads();
	}

	@Override
	protected boolean handleMessage(int tag, int length, SafeReader source) throws IOException {
		setTimeout(CONFIG.BROADCAST_TIMEOUT);
		switch (tag){
		case MESSAGE_HELLO:   handleHello(HelloMessage.getParser().parseMessage(tag, length, source)); return true;
		case MESSAGE_STATE:   handleState(ChannelMessage.getParser().parseMessage(tag, length, source)); return true;
		case MESSAGE_REQUEST: handleRequest(ChannelMessage.getParser().parseMessage(tag, length, source)); return true;
		case MESSAGE_PAYLOAD: handlePayload(PayloadMessage.getParser().parseMessage(tag, length, source)); return true;
		case MESSAGE_END_OF_DATA:  handleEndOfData(EndOfDataMessage.getParser().parseMessage(tag, length, source)); return true;
		case MESSAGE_FINISHED:  handleFinished(); return true;
		case MESSAGE_DEBUG:   handleDebug(DebugMessage.getParser().parseMessage(tag, length, source)); return true;
		default: throw new ProtocolException("Unknown message tag");
		}
	}

	private void handleHello(HelloMessage hello) throws ProtocolException {
		log("Received", "HELLO", CryptoHelper.longToHex(hello.getNodeId()), Integer.toHexString(hello.getConnectionId()));
		
		if (remoteNodeId != hello.getNodeId()) {
			log("node ID mismatch", CryptoHelper.longToHex(remoteNodeId),
						CryptoHelper.longToHex(hello.getNodeId()));
			close();
			return;
		}
		
		if (hello.getVersion() < CONFIG.BROADCAST_MIN_VERSION) {
			log("remote version too old", hello.getMinVersion(), hello.getVersion());
			close();
			return;
		}
		
		if (hello.getMinVersion() > manager.getVersion()) {
			log("local version too old", hello.getMinVersion(), manager.getVersion());
			close();
			return;
		}
		
		sentInstanceId  = manager.getState().getInstanceId();
		sentTimestamp = SystemClock.elapsedRealtime();
		
		ArrayList<ChannelState> offer = manager.getState().getOffer(hello.getInstanceId(), hello.getTimestamp());
		log("Sending", "STATE", offer.size());
		sendMessage(new ChannelMessage(MESSAGE_STATE,offer));
	}

	private void handleState(ChannelMessage msg) throws ProtocolException {
		log("Received", "STATE", msg.getChannels().size());
		ArrayList<ChannelState> req = manager.getState().getMessagesToRequest(msg.getChannels());
		Collections.shuffle(req);
		log("Sending", "REQUEST", req.size());
		sendMessage(new ChannelMessage(MESSAGE_REQUEST, req));
	}

	private void handleRequest(ChannelMessage msg) {
		log("Received", "REQUEST", msg.getChannels().size());
		for (ChannelState channel : msg.getChannels()) {
			for (int msgId : channel.getMessageIDs()) {
				log("Sending", "PAYLOAD", CryptoHelper.longToHex(channel.getChannelId()), msgId);
				sendMessage(new PayloadMessage(channel.getChannelId(), msgId, CONFIG.BROADCAST_PAYLOAD_SIZE));
			}
		}
		log("Sending", "ENDOFDATA");
		sendMessage(new EndOfDataMessage(sentInstanceId, sentTimestamp));
	}

	private void handlePayload(PayloadMessage msg) {
		log("Received", "PAYLOAD", CryptoHelper.longToHex(msg.getChannelId()), msg.getMessageId());
		manager.getState().addMessage(msg.getChannelId(), msg.getMessageId());
	}

	private void handleEndOfData(EndOfDataMessage msg) {
		log("Received", "ENDOFDATA", msg.getInstanceId(), msg.getTimestamp());
		manager.completedTransaction(remoteNodeId, msg.getInstanceId(), msg.getTimestamp());
		log("Sending", "FINISHED");
		sendMessage(new FinishedMessage());
		oneSideFinished();
	}
	
	private void handleFinished() {
		log("Received", "FINISHED");
		oneSideFinished();
	}

	private void handleDebug(DebugMessage msg) {
		log("Received", "DEBUG", msg.getString());
	}

	/** 
	 * Called when one of the two protocol instances is done
	 * (i.e. once when the "End of data" message has been received
	 *       once when the "Finished" message has been received).
	 * On first invocation, does nothing.
	 * On second invocation, calls done();
	 */
	private synchronized void oneSideFinished() {
		if (!oneSideFinished) {
			oneSideFinished = true;
		} else {
			done();
		}
	}

	@Override
	protected void initiateConnection() {
		PeerInfo peer = manager.getPeerInfo(remoteNodeId);
		sendMessage(new HelloMessage(manager.getMyId(), CONFIG.BROADCAST_MIN_VERSION, manager.getVersion(),
				getConnID(), peer.instanceId, peer.timestamp));
	}

	
	public static final int MESSAGE_HELLO   =    1;
	public static final int MESSAGE_STATE   =   11;
	public static final int MESSAGE_REQUEST =   21;
	public static final int MESSAGE_PAYLOAD =   31;
	public static final int MESSAGE_END_OF_DATA  = 98;
	public static final int MESSAGE_FINISHED     = 99;
	public static final int MESSAGE_DEBUG     = 1000;

}
